/* *****************************************************************************
 * platform/comlist/linux.c -- find out Linux serial devices.
 * Copyright (C) 2016-2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libp7.
 * libp7 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libp7 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libp7; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libp7/internals.h>
#ifdef __linux__
# include <stdlib.h>
# include <string.h>
# include <sys/stat.h>
# include <dirent.h>
# include <fcntl.h>
# include <unistd.h>
# include <errno.h>

/**
 *	p7_comlist_linux:
 *	List serial devices under Linux.
 *
 *	Links in the /dev/serial/by-id/ should resolve to relative paths, but
 *	I also managed absolute paths, in case.
 *
 *	@arg	callback		the callback.
 *	@arg	cookie			the cookie.
 *	@return					the error.
 */

int p7_comlist_linux(p7_list_device_t callback, void *cookie)
{
	/* open the thing */
	char path[PATH_MAX + 1]; strcpy(path, "/dev/serial/by-id/");
	DIR *d = opendir(path);
	if (!d) return (0);

	/* prepare */
	char *f = strchr(path, 0), devname[PATH_MAX + 1];

	/* read the entries */
	struct dirent *dr; struct stat st;
	while ((dr = readdir(d))) {
		/* check type */
		strcpy(f, dr->d_name);
		if (lstat(path, &st) || (st.st_mode & S_IFMT) != S_IFLNK)
			continue;

		/* get destination path and send it */
		ssize_t rl = readlink(path, devname, PATH_MAX + 1);
		if (rl < 0) continue;
		devname[rl] = 0;
		if (devname[0] == '/')
			(*callback)(cookie, devname);
		else {
			strcpy(f, devname);
			char *p = realpath(path, devname);
			if (!p) continue;
			(*callback)(cookie, p);
		}
	}

	/* close the dir and we're done listing */
	closedir(d);
	return (0);
}

# endif /* __linux__ */
