/* *****************************************************************************
 * platform/comlist/windows.c -- find out serial devices on MS-Windows.
 * Copyright (C) 2016-2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libp7.
 * libp7 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libp7 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libp7; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libp7/internals.h>
#ifndef P7_DISABLED_WINDOWS
/* only works because I redefined the version at the beginning
 * of `internals.h`! */
# include <windows.h>
# include <setupapi.h>
# include <usbiodef.h>
# include <winerror.h>

/**
 *	p7_comlist_windows:
 *	List serial devices under MS-Windows.
 *
 *	I'm using the source from Python 3.x's winreg module source and the MSDN
 *	documentation.
 *
 *	@arg	callback	the callback to which to send the path.
 *	@arg	cookie		the cookie to pass to the callback.
 *	@return				the error code (0 if ok).
 */

int p7_comlist_windows(p7_list_device_t callback, void *cookie)
{
	HKEY hkey; int hkey_open = 0;
	char *value = NULL, *data = NULL;

	/* read the registry key */
	logr_info("Opening HKEY_LOCAL_MACHINE\\HARDWARE\\DEVICEMAP\\SERIALCOMM");
	DWORD werr = RegOpenKey(HKEY_LOCAL_MACHINE,
		"HARDWARE\\DEVICEMAP\\SERIALCOMM", &hkey);
	if (werr) goto fail;
	hkey_open = 1;

	/* prepare */
	logr_info("Allocating enough space");
	DWORD valsize = 1024, datasize = 1024; /* I DON'T CARE IT WORKS LALALA */
	if (!(value = malloc(valsize)) || !(data = malloc(datasize)))
		goto fail;

	/* enumerate values */
	logr_info("Enumerating values");
	DWORD type, curval = valsize, curdata = datasize;
	for (DWORD i = 0; (werr = RegEnumValue(hkey, i, value, &curval, NULL,
	  &type, (BYTE*)data, &curdata)) != ERROR_NO_MORE_ITEMS; i++) {
		if (type != REG_SZ)
			continue;
		if (werr == ERROR_MORE_DATA) {
			logr_error("data not big enough, o shit waddap");
			goto fail;
		}

		(*callback)(cookie, data);
		curval = valsize;
		curdata = datasize;
	}

	/* error is `ERROR_MORE_DATA`, o ship shaddap */
	werr = 0;

fail:
# if LOGLEVEL <= ll_error
	if (werr) logr_error("Got error %08lu", werr);
# endif
	/* free, close the key and return ! */
	free(value); free(data);
	if (hkey_open) RegCloseKey(hkey);
	return (werr ? p7_error_unknown : 0);
}

#endif /* __WINDOWS__ */
